/*	Filename:		BustaEngine.h
 *	Description:	Busta' Sandwich engine module.  Interfaces with the graphics and networking modules.
 *
 */

#ifndef BUSTA_SANDWICH_ENGINE
#define BUSTA_SANDWICH_ENGINE

#include <stdio.h>
#include <string>
#include "../graphics/BustaGraphics.h"
#include "../networking/BustaNetClient.h"
#include "../Models/BustaModelGameState.h"
#include "../Models/BustaModelCompleteSandwich.h"

using namespace std;

#define ROTATE SDLK_j
#define SWAP SDLK_k
#define DOWN SDLK_DOWN
#define UP SDLK_UP
#define RIGHT SDLK_RIGHT
#define LEFT SDLK_LEFT
#define QUIT SDLK_ESCAPE
#define SUBMIT SDLK_SPACE
#define SWITCH_SUBMIT SDLK_RALT

/**
 *@enum ScreenState
 *@brief State that the engine is currently in
 *
 * Current state the engine/screen is in
 */
enum ScreenState{MAIN_MENU,OPTIONS,HIGH_SCORES,CHAR_SELECT,SP_GAME,CONNECT,MP_GAME,MP_RANKING,KEY_BINDINGS,SOUNDS,DIFFICULTY,EXIT};


/**
 * @class BustaEngine
 * @brief Busta'Sandwich Engine
 *
 * Busta'Sandwich Engine for single player and networked multiplayer games.
 */

class BustaEngine
{
	public:

		/**
		 * @brief Constructor
		 *
		 * Initializes variables
		 */
		BustaEngine();

		/**
		 * @brief Destructor
		 *
		 */
		~BustaEngine();

		/**
		 * @brief Gets the current/next screen state
		 *
		 * Gets the engine (screen) state
		 *
		 * @return Returns a ScreenState describing which screen the engine should be in
		 */
		ScreenState getState();

		/**
		 * @brief Sets current/next screen state
		 *
		 * sets the engine (screen) state
		 */
		void setState(ScreenState s);

		/**
		 * @brief Main Menu
		 *
		 * Displays main menu, returns when moving to new menu
		 *
		 */
		void mainMenu();

		/**
		 * @brief Options Menu
		 *
		 * Displays the options menu, returns when moving to new menu
		 * Currently a stub function, sets back to main menu!
		 *
		 */
		void optionsMenu();

		/**
		 * @brief High Scores Screen
		 *
		 * Displays the high scores, returns when moving to main menu
		 *
		 */
		void highScores();

		/**
		 * @brief Character Select Screen
		 *
		 * Displays the character select screen, allowing user(s) to choose
		 * character
		 * Currently a stub function, sets to SP_GAME!
		 *
		 */
		void characterSelect();

		/**
		 * @brief Connection Screen
		 *
		 * Displays the connection screen and handles the multiplayer connection
		 * Currently a stub function, sets to CHAR_SELECT!
		 *
		 */
		void connectScreen();

		/**
		 * @brief Key Binding Menu
		 *
		 * Sub-menu under options, allows user to change the key bindings of the game
		 * Currently a stub function that sends back to Options menu
		 *
		 */
		void keyBindingMenu();

		/**
		 * @brief Difficulty Screen
		 *
		 * Sub-menu under options, allows user to change the sound options
		 * Currently a stub function that sends back to Options menu
		 *
		 */
		void soundsMenu();

		/**
		 * @brief Difficulty Screen
		 *
		 * Sub-menu under options, allows user to change the difficulty of the game
		 * Currently a stub function that sends back to Options menu
		 *
		 */
		void difficultyMenu();


		/**
		 * @brief SP Game
		 *
		 * Displays and runs the game loop (SP or MP), returns when game over for 
		 * single player, when multiplayer is over, or when explicitly exit
		 *
		 */
		void playGame();

		/**
		 * @brief Ranking Screen
		 *
		 * Displays the ranking of the players at the end of a multiplayer game
		 * Currently a stub function, sets state to MAIN_MENU!
		 *
		 */
		void displayRankings();

		/**
		 * @brief Initializes network connection
		 *
		 * Initializes the network connection, returns error messages
		 *
		 * @return Returns a char* describing any errors that occured; returns NULL if connected correctly
		 */
		char* netInit(char* ip);

		int moveLeft(int col_curr, int align_curr);
		int moveRight(int col_curr, int align_curr);
		int moveUp(int row_curr, int align_curr);
		int moveDown(int row_curr, int align_curr);
		int swap(int row_curr, int col_curr, int align_curr); //switch pieces
		int rotate(int row_curr, int col_curr, int align_curr); //change axis of rotation
		int submit_cus();
		int submit_self();
		void submit_board();
		BustaModelGameState* getGameState();




	private:
		int pause;
		int interval;
		bool multiplayer;
		BustaNetClient* network;
	
		BustaGraphics* graphics;
		BustaModelGameState* gs;
		ScreenState state;

		/**
		 * @brief Returns true if x,y coordinate over button area
		 *
		 * Returns true if x,y coordinate over button area
		 */
		bool buttonSelected(Button b, int x, int y);

}; 

#endif
