
#ifndef BUSTASANDWICH_INTERNAL_MESSAGES
#define BUSTASANDWICH_INTERNAL_MESSAGES

#include "../models/BustaModelSpecialSandwich.h"
#include "../models/BustaModelPosition.h"
#include <stdlib.h>

/**
 * @struct netClient BustaNetMessages.h
 *
 * @brief A struct representation of client information
 *
 * This representation of a client contains information about the client's network identification
 *	number and the client's player's name.
 */
struct netClient
{
	int clientID;		/**< The identification number of the client */
	char* clientName;	/**< The name of the client's player */
};

/**
 * @class BustaNetInternalMsg
 * @brief Base class for all internal network messages
 *
 * This is the base class for all messages that are sent between the client and the game engine.
 */
class BustaNetInternalMsg
{
	public:
		/**
		 * @brief Returns the type of the message.
		 *
		 * The type of the internal message is returned.  This type identifies messages from one another
		 *	and allows the engine to determine the action to take with the message.
		 *
		 * @return The integer representation of a message type.
		 */
		int getType();

		/**
		 * @brief Sets the type of the message
		 *
		 * Sets the identification type of the message.
		 *
		 * @param newType The new int type identifier for the message
		 */
		void setType(int newType);

	protected:
		int type;	// The type of the message
};

/**
 * @class MPSpecialSandwich
 * @brief Special Sandwich send message (Type: 301)
 *
 * This message defines the details of a special sandwich when it is sent from one player to another.
 *	When a player receives this message he/she is able to determine who sent a special sandwich, who is
 *	receiving that special sandwich, and what the actual special sandwich is.
 */
class MPSpecialSandwich : public BustaNetInternalMsg
{
	public:
		/**
		 * @brief CONSTRUCTOR
		 *
		 * Creates an instance of the special sandwich message.
		 *
		 * @param sendID The identification number of the sandwich sender
		 * @param receiveID The identification number of the sandwich receiver
		 * @param sSandwich The special sandwich sent between the players
		 */
		MPSpecialSandwich(int sendID, int receiveID, SpecialSandwich sSandwich);

		/**
		 * @brief Returns the identification number of the player sending the sandwich
		 *
		 * Returns an integer identification number of the player sending the special sandwich to
		 *	an opponent.
		 *
		 * @return The identification number of the sending player
		 */
		int getSenderID();

		/**
		 * @brief Returns the identification number of the player receiving the sandwich
		 *
		 * Returns an integer identification number of the player receiving the special sandwich
		 *	from the sending player.
		 *
		 * @return The identification number of the receiving player
		 */
		int getReceiverID();

		/**
		 * @brief Returns the special sandwich sent
		 *
		 * Returns an enumeration of the special sandwich sent between players.
		 *
		 * @return The special sandiwch sent
		 */
		SpecialSandwich getSpecialSandwich();

	private:
		int senderID;		// identification number of the player sending the sandwich
		int receiverID;		// identification number of the player receiving the sandwich
		SpecialSandwich specialSandwich;	// the special sandwich being sent
};

/**
 * @class MPCharSelected
 * @brief MP game character selected message (Type: 101)
 *
 * This message is received by all players when another player in a multiplayer game has made a character
 *	selection.  The player can then tell who has made the selection and which character was selected.
 */
class MPCharSelected : public BustaNetInternalMsg
{
	public:
		/**
		 * @brief CONSTRUCTOR
		 *
		 * Creates an instance of the MPCharSelected class.  Upon creation, the message details are
		 *	specified.
		 *
		 * @param pID The identification number of the player who has selected a character
		 * @param selectedChar The character that was selected by the player
		 */
		MPCharSelected(int pID, char selectedChar);

		/**
		 * @brief Returns the identification number of the player who has selected a character
		 *
		 * An integer identification number of the player who has made a character selection is
		 *	returned.
		 *
		 * @return The identification number of the player who has selected a character
		 */
		int getPlayerID();

		/**
		 * @brief Returns the character the player has selected
		 *
		 * Returns an char representation of the character that the player has selected.
		 *
		 * @return A char representation of the selected character
		 */
		char getSelectedChar();

	private:
		int playerID;		// the identification number of the player who selected a character
		char character;		// the character that the player selected
};

/**
 * @class MPCharSelectCursor
 * @brief MP game character selection cursor message (Type: 102)
 *
 * This message is received by all players when another player in a multiplayer game moves their character
 *	selection cursor.  Each player is able to see the character selection of another client during
 *	the character selection phase of a MP game.  Having this message, the player can tell who's moving their
 *	cursor and that cursor's position.
 */
class MPCharSelectCursor : public BustaNetInternalMsg
{
	public:
		/**
		 * @brief CONSTRUCTOR
		 *
		 * An instance of the MPCharSelectCursor is created.  Information to be contained in the message
		 *	is necessarily supplied at creation time.
		 *
		 * @rparam pID The identification number of the player whose cursor is being sent
		 * @param pCursor The position of the player's cursor
		 */
		MPCharSelectCursor(int pID, BustaModelPosition pCursor);

		/**
		 * @brief Returns the identification number of the player who has moved their cursor
		 *
		 * An integer identification number of the player who has made a cursor movement is returned.
		 *
		 * @return The identification number of the player who has moved their cursor
		 */
		int getPlayerID();

		/**
		 * @brief Returns the position of a multiplayer player's cursor during character selection
		 *
		 * Returns the position of a multiplayer player's cursor during character selection
		 *
		 * @return Returns the position of a multiplayer player's cursor during character selection
		 */
		BustaModelPosition getCursor();

	private:
		int playerID;						// the identification number of the player who moved their cursor
		BustaModelPosition playerCursor;	// the position of the player's cursor
};

/**
 * @class MPCharSelectMode
 * @brief MP game character selection phase message (Type: 201)
 *
 * This message is received by all players when the multiplayer game has gone into character selection
 *	mode.  This mode follows directly after the connection phase.  In this phase players are allowed
 *	to make their character selections.
 */
class MPCharSelectMode : public BustaNetInternalMsg
{
	public:
		/**
		 * @brief CONSTRUCTOR
		 *
		 * Creates an instance of the MPCharSelectMode class.
		 */
		MPCharSelectMode();
};

/**
 * @class MPStartGame
 * @brief MP game start message (Type: 202)
 *
 * This message is received by all players when the multiplayer game is to begin.  This mode follows
 *	directly after the character selection phase.
 */
class MPStartGame : public BustaNetInternalMsg
{
	public:
		/**
		 * @brief CONSTRUCTOR
		 *
		 * Creates an instance of the MPStartGame class.
		 */
		MPStartGame();
};

/**
 * @class MPEndGame
 * @brief MP game end message (Type: 203)
 *
 * This message is received by all players when the multiplayer game has ended.
 */
class MPEndGame : public BustaNetInternalMsg
{
	public:
		/**
		 * @brief CONSTRUCTOR
		 *
		 * Creates an instance of the MPEndGame class.
		 */
		MPEndGame();
};

/**
 * @class MPPlayerLost
 * @brief MP Game player lost message (Type: 204)
 *
 * This message is received when a player has lost in the current multiplayer game.  Players who receive
 *	this message are notified of the identification number of the losing player.
 */
class MPPlayerLost : public BustaNetInternalMsg
{
	public:
		/**
		 * @brief CONSTRUCTOR
		 *
		 * Creates an instance of the MPPlayerLost class.
		 *
		 * @param pID The identification number of the losing player
		 */
		MPPlayerLost(int pID);

		/**
		 * @brief Returns the identification number of the losing player
		 *
		 * Returns the integer identification number of the player who has lost in the current 
		 *	multiplayer game.
		 *
		 * @return The identification number of the losing player
		 */
		int getPlayerID();

	private:
		int playerID;		// the identification number of the player who lost
};

/**
 * @class MPConnectResponse
 * @brief MP Game join game server response (Type: 1)
 *
 * This message is received when a player has attempted to connected to a game server.  The server responds
 *	with an acceptance or denial to the client's connections attempt.  The information about that response
 *	is encapsulated in this message.
 */
class MPConnectResponse : public BustaNetInternalMsg
{
	public:
		/**
		 * @brief CONSTRUCTOR
		 *
		 * Creates an instance of the MPConnectResponse class.
		 *
		 * @param accept Whether or not the client was allowed (true for allowed)
		 * @param msg The reason for denial if denied
		 * @param cID The client's identification number if accepted
		 */
		MPConnectResponse(bool accept, char* msg, int cID);

		/**
		 * @brief Returns the acceptance by the server
		 *
		 * Returns whether or not the server allowed the client to join the game.  An acceptance of
		 *	true denotes that the server allowed the client to join the game.  A return of false denotes
		 *	denial of the client's join game request.
		 *
		 * @return The server's acceptance of the client (true for allowed, false for denied)
		 */
		bool getAcceptance();

		/**
		 * @brief Returns the reason for denial by the server
		 *
		 * This method returns the reason why the server has denied the client if the client was denied.
		 *
		 * @return The reason for server acceptance response
		 */
		char* getReason();

		/**
		 * @brief Returns the client's identification number if accepted
		 *
		 * The server provides the client with an identification number for the multiplayer game if the
		 *	the client was allowed to join the game.  This method returns that identification number.
		 *
		 * @return The identification number assigned to the client by the server if accepted
		 */
		int getClientID();

	private:
		bool acceptance;	// whether or not the client is allowed to connect (true for allowed)
		char* reason;		// the reason for denial of connection if denied	
		int clientID;		// the client's identification number of accepted
};

/**
 * @class MPNewPlayer
 * @brief MP Game new player joined message (Type: 2)
 *
 * This message is received when a new player has joined the multiplayer game.  When a client receives this
 *	message they are notified of the new player's identification number and name.
 */
class MPNewPlayer : public BustaNetInternalMsg
{
	public:
		/**
		 * @brief CONSTRUCTOR
		 *
		 * Creates an instance of the MPNewPlayer class.
		 *
		 * @param pID The identification number of the new player
		 * @param pName The name of the new player
		 */
		MPNewPlayer(int pID, char* pName);

		/**
		 * @brief Returns the new player's identification number
		 *
		 * Returns the integer identification number of the player who has just joined the game.
		 *
		 * @return The identification number of the new player
		 */
		int getPlayerID();

		/**
		 * @brief Returns the new player's name
		 *
		 * Returns the name of the new player.
		 *
		 * @return The name of the new player
		 */
		char* getPlayerName();

	private:
		int playerID;		// the identification number of the new player
		char* playerName;	// the name of the new player
};

/**
 * @class MPPlayerDisconnect
 * @brief MP Game player disconnection message (Type: 3)
 *
 * This message is received when a player has disconnected from the multiplayer game.  Receivers of
 *	the message are told the identification number of the disconnected player as well as the reason
 *	for disconnection.
 */
class MPPlayerDisconnect : public BustaNetInternalMsg
{
	public:
		/**
		 * @brief CONSTRUCTOR
		 *
		 * Creates an instance of the MPPlayerDisconnect class.
		 *
		 * @param pID The identification number of the disconnected player
		 * @param msg The reason for the player's disconnection
		 */
		MPPlayerDisconnect(int pID, char* msg);

		/**
		 * @brief Returns the disconnected player's identification number
		 *
		 * Returns the integer identification number of the player who has disconnected
		 *
		 * @return The identification number of the disconnected player
		 */
		int getPlayerID();

		/**
		 * @brief Returns the reason for the player's disconnection
		 *
		 * Returns the reason for disconnection by the player.  This could simply be that the player
		 *	has decided to leave the game, the player has timed-out from the server, etc.
		 *
		 * @return The reason for disconnection
		 */
		char* getReason();

	private:
		int playerID;	// the identification number of the disconnected player
		char* reason;	// the reason for the player's disconnection
};

/**
 * @class MPServerSettings
 * @brief MP Game Server Settings message (Type: 4)
 *
 * This message contains information about the game server.  This includes the server name, the 
 *	number of connected clients, and client information.
 */
class MPServerSettings : public BustaNetInternalMsg
{
	public:
		/**
		 * @brief CONSTRUCTOR
		 *
		 * Creates an instance of the MPServerSettings class.
		 *
		 * @param sName The name of the server
		 * @param nClients The number of currently connect clients
		 * @param sClients An array of netClients containing information about the connected clients
		 */
		MPServerSettings(char* sName, int nClients, netClient** sClients);

		/**
		 * @brief DESTRUCTOR
		 *
		 * Frees memory associated with message.
		 */
		~MPServerSettings();

		/**
		 * @brief Returns the name of the server
		 *
		 * Each game server is allowed to have a specific name.  This method returns the name of the server
		 *	when a MPServerSettings message is received.
		 *
		 * @return The server name
		 */
		char* getServerName();

		/**
		 * @brief Returns the number of connected clients
		 *
		 * Returns the number of clients connected to the game server.
		 *
		 * @return The number of connected clients
		 */
		int getNumClients();

		/**
		 * @brief Returns a connected client's information
		 *
		 * The server maintains a listing of all clients connected to it.  This method returns the
		 *	information about a single one of those clients.
		 *
		 * @param index The index of the client whose information to return
		 *
		 * @return A netClient pointer containing connected client information.  NULL is returned if
		 *	the requested index is negative or is greater than numClients, the number of currently
		 *	connected clients.
		 *
		 * @see netClient
		 */
		netClient* getClient(int index);

		/**
		 * @brief Returns connected client information
		 *
		 * The server maintains a listing of all clients connected to it.  This method returns the
		 *	information about those clients in an array.
		 *
		 * @see netClient
		 *
		 * @return An array of netClients containing connected client information
		 */
		netClient** getClients();

		/**
		 * @brief Sets the recorded server name
		 *
		 * Sets the client's perception of what the server's name is to the client.
		 *
		 * @param newName The new name to be associated with the server
		 */
		void setServerName(char* newName);

	private:
		char* serverName;		// name of the server
		int numClients;			// number of connected clients to the server
		netClient** clients;	// array of netClients containing information about connected clients
};

#endif
